/**
 * Copyright  2008 Kevin Twidle, Imperial College, London, England.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * Contact: Kevin Twidle <kpt@doc.ic.ac.uk>
 *
 * Created on Jul 29, 2009
 *
 * $Log:$
 */

package net.ponder2.managedobject;

import net.ponder2.ManagedObject;
import net.ponder2.apt.Ponder2op;
import net.ponder2.exception.Ponder2ArgumentException;
import net.ponder2.exception.Ponder2Exception;
import net.ponder2.exception.Ponder2OperationException;
import net.ponder2.objects.P2Object;

/**
 * TODO Description
 * 
 * @author Kevin Twidle
 * @version $Id:$
 */
public class Cart implements ManagedObject {

	private P2Object percepts;
	private boolean loaded;
	int x, y;
	String facing;

	@Ponder2op("create:")
	Cart(P2Object percepts) {
		this.percepts = percepts;
		loaded = false;
	}

	@Ponder2op("setFacing:x:y:")
	protected void setup(String facing, int x, int y) {
		this.facing = facing;
		this.x = x;
		this.y = y;
	}

	@Ponder2op("loaded")
	boolean loaded() {
		return loaded;
	}

	@Ponder2op("gsc")
	protected boolean gsc(P2Object source) throws Ponder2ArgumentException,
			Ponder2Exception {
		P2Object mine = percepts.operation(source, "at:", "mine");
		return mine.operation(source, "gscx:gscy:", "" + x, "" + y).asBoolean();
	}

	@Ponder2op("pickup")
	void pickup() {
		loaded = true;
	}

	@Ponder2op("putout")
	void putout() {
		loaded = false;
	}

	// Facing stuff
	@Ponder2op("facingN")
	boolean facingNorth(P2Object source) throws Ponder2ArgumentException,
			Ponder2OperationException, Ponder2Exception {
		return facing.equals("n");
	}

	@Ponder2op("facingE")
	boolean FacingSouth(P2Object source) throws Ponder2ArgumentException,
			Ponder2OperationException, Ponder2Exception {
		return facing.equals("e");
	}

	@Ponder2op("facingS")
	boolean facingEast(P2Object source) throws Ponder2ArgumentException,
			Ponder2OperationException, Ponder2Exception {
		return facing.equals("s");
	}

	@Ponder2op("facingW")
	boolean facingWest(P2Object source) throws Ponder2ArgumentException,
			Ponder2OperationException, Ponder2Exception {
		return facing.equals("w");
	}

	// Wall stuff

	@Ponder2op("wallL")
	boolean wallLeft(P2Object source) throws Ponder2ArgumentException,
			Ponder2OperationException, Ponder2Exception {
		String dir = left(facing);
		return wall(source, dir);
	}

	@Ponder2op("wallF")
	boolean wallFront(P2Object source) throws Ponder2ArgumentException,
			Ponder2OperationException, Ponder2Exception {
		String dir = facing;
		return wall(source, dir);
	}

	@Ponder2op("wallR")
	boolean wallRight(P2Object source) throws Ponder2ArgumentException,
			Ponder2OperationException, Ponder2Exception {
		String dir = right(facing);
		return wall(source, dir);
	}

	private boolean wall(P2Object source, String dir) throws Ponder2Exception,
			Ponder2ArgumentException {
		P2Object mine = percepts.operation(source, "at:", "mine");
		return mine.operation(source, "wallFacing:x:y:", dir, "" + x, "" + y)
				.asBoolean();
	}

	// Move

	@Ponder2op("moveF")
	void move(P2Object source) throws Ponder2ArgumentException,
			Ponder2OperationException, Ponder2Exception {
		// System.out.format("Moving from (%d,%d)", x, y);
		char dir = facing.charAt(0);
		switch (dir) {
		case 'n':
			move(source, 0, -1);
			break;
		case 'e':
			move(source, 1, 0);
			break;
		case 's':
			move(source, 0, 1);
			break;
		case 'w':
			move(source, -1, 0);
			break;
		}
		// System.out.format(" to (%d,%d)", x, y);
		// System.out.println();
	}

	private void move(P2Object source, int dx, int dy)
			throws Ponder2ArgumentException, Ponder2OperationException,
			Ponder2Exception {
		if (!wallFront(source)) {
			x = x + dx;
			y = y + dy;
		}
	}

	@Ponder2op("turnL")
	void turnLeft(P2Object source) throws Ponder2ArgumentException,
			Ponder2OperationException, Ponder2Exception {
		facing = left(facing);
	}

	@Ponder2op("turnR")
	void turnRight(P2Object source) throws Ponder2ArgumentException,
			Ponder2OperationException, Ponder2Exception {
		facing = right(facing);
	}

	// Direction helpers

	public static String left(String dir) {
		String newdir = null;
		switch (dir.charAt(0)) {
		case 'n':
			newdir = "w";
			break;
		case 'e':
			newdir = "n";
			break;
		case 's':
			newdir = "e";
			break;
		case 'w':
			newdir = "s";
			break;
		}
		return newdir;
	}

	public static String right(String dir) {
		String newdir = null;
		switch (dir.charAt(0)) {
		case 'n':
			newdir = "e";
			break;
		case 'e':
			newdir = "s";
			break;
		case 's':
			newdir = "w";
			break;
		case 'w':
			newdir = "n";
			break;
		}
		return newdir;
	}

	@Ponder2op("wait")
	void waitForABit() {
		try {
			System.out.println("Cart waiting");
			Thread.sleep(1000);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}
