/**
 * Copyright  2008 Kevin Twidle, Imperial College, London, England.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * Contact: Kevin Twidle <kpt@doc.ic.ac.uk>
 *
 * Created on Aug 7, 2009
 *
 * $Log:$
 */

package net.ponder2.managedobject;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import net.ponder2.ManagedObject;
import net.ponder2.apt.Ponder2op;
import net.ponder2.exception.Ponder2Exception;
import net.ponder2.objects.P2Object;

/**
 * TODO Description
 * 
 * @author Kevin Twidle
 * @version $Id:$
 */
public class PolicyPercepts extends Percepts implements ManagedObject {
	
	Map<String,List<P2Object>> policySets;
	List<P2Object> currentSet;
	
	@Ponder2op("create")
	PolicyPercepts() {
		policySets = new HashMap<String, List<P2Object>>();
		currentSet = null;
	}
	
	@Ponder2op("at:add:")
	protected void atadd(P2Object source, String aName, P2Object aPolicy) throws Ponder2Exception {
		if (!policySets.containsKey(aName)) {
			policySets.put(aName, new Vector<P2Object>());
		}
		List<P2Object> policySet = policySets.get(aName);
		policySet.add(aPolicy);
		// If currently active set enable the policy
		// TODO If not active should we disable the policy?
		// Probably not, it may also be in the active set
		if (policySet == currentSet)
			aPolicy.operation(source, "active:", P2Object.create(true));
	}
	
	@Ponder2op("enable:")
	protected void enable(P2Object source, String aName) throws Ponder2Exception {
		// Remove old policies
		if (currentSet != null) {
			setPolicies(source, currentSet, false);
		}
		// Start new policies
		currentSet = policySets.get(aName);
		setPolicies(source, currentSet, true);
		// Let the TR rulesets know of the change
		event(source);
	}
	
	@Ponder2op("disable")
	protected void disable(P2Object source) throws Ponder2Exception {
		if (currentSet != null) {
			setPolicies(source, currentSet, false);
			currentSet = null;
		}
	}

	/**
	 * @param source 
	 * @param policySet
	 * @param active
	 * @throws Ponder2Exception 
	 */
	private void setPolicies(P2Object source, List<P2Object> policySet, boolean active) throws Ponder2Exception {
		for (P2Object policy : policySet) {
			policy.operation(source, "active:", P2Object.create(active));
		}
		
	}

}
